import { __rest, __assign } from 'tslib';
import { createContext, useContext, useMemo, createElement } from 'react';
import { loadFeatures } from '../motion/features/definitions.js';
import { useConstant } from '../utils/use-constant.js';

/**
 * @public
 */
var MotionConfigContext = createContext({
    transformPagePoint: function (p) { return p; },
    isStatic: false,
});
/**
 * MotionConfig can be used in combination with the `m` component to cut bundle size
 * and dynamically load only the features you use.
 *
 * ```jsx
 * import {
 *   m as motion,
 *   AnimationFeature,
 *   MotionConfig
 * } from "framer-motion"
 *
 * export function App() {
 *   return (
 *     <MotionConfig features={[AnimationFeature]}>
 *       <motion.div animate={{ x: 100 }} />
 *     </MotionConfig>
 *   )
 * }
 * ```
 *
 * @public
 */
function MotionConfig(_a) {
    var children = _a.children, isStatic = _a.isStatic, config = __rest(_a, ["children", "isStatic"]);
    var parentConfig = useContext(MotionConfigContext);
    /**
     * Don't allow isStatic to change between renders as it affects how many hooks
     * motion components fire.
     */
    isStatic = useConstant(function () { return isStatic; });
    /**
     * Inherit and overwrite any parent MotionConfigs
     */
    config = __assign(__assign({}, parentConfig), config);
    var allDefinedFeatures = __assign(__assign({}, parentConfig.features), config.features);
    var loadedFeatures = Object.keys(allDefinedFeatures).filter(function (name) { return allDefinedFeatures[name] !== null; });
    var numLoadedFeatures = loadedFeatures.length;
    /**
     * Creating a new config context object will re-render every `motion` component
     * every render. So we only create a new one when we *want* to rerender these
     * components, for instance when a new feature has loaded, or the renderer has loaded.
     */
    var context = useMemo(function () {
        // TODO We probably don't need to pass this through context
        config.features && loadFeatures(config.features);
        return __assign(__assign({}, config), { features: allDefinedFeatures, isStatic: isStatic });
    }, [numLoadedFeatures, config.renderer, config.transition]);
    // Mutative to prevent triggering rerenders in all listening
    // components every time this component renders
    context.transformPagePoint = config.transformPagePoint;
    return (createElement(MotionConfigContext.Provider, { value: context }, children));
}

export { MotionConfig, MotionConfigContext };
